@@ -4,7 +4,7 @@ from esphome import core
 from esphome.components import display
 import esphome.config_validation as cv
 import esphome.codegen as cg
-from esphome.const import CONF_FILE, CONF_GLYPHS, CONF_ID, CONF_SIZE
+from esphome.const import CONF_FILE, CONF_GLYPHS, CONF_ID, CONF_RAW_DATA_ID, CONF_SIZE
 from esphome.core import CORE, HexInt
 
 DEPENDENCIES = ["display"]
@@ -12,6 +12,7 @@ MULTI_CONF = True
 
 Font = display.display_ns.class_("Font")
 Glyph = display.display_ns.class_("Glyph")
+GlyphData = display.display_ns.struct("GlyphData")
 
 
 def validate_glyphs(value):
@@ -60,8 +61,7 @@ def validate_pillow_installed(value):
 def validate_truetype_file(value):
     if value.endswith(".zip"):  # for Google Fonts downloads
         raise cv.Invalid(
-            "Please unzip the font archive '{}' first and then use the .ttf files "
-            "inside.".format(value)
+            f"Please unzip the font archive '{value}' first and then use the .ttf files inside."
         )
     if not value.endswith(".ttf"):
         raise cv.Invalid(
@@ -72,9 +72,9 @@ def validate_truetype_file(value):
 
 
 DEFAULT_GLYPHS = (
-    ' !"%()+,-.:/0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz°'
+    ' !"%()+=,-.:/0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZ_abcdefghijklmnopqrstuvwxyz°'
 )
-CONF_RAW_DATA_ID = "raw_data_id"
+CONF_RAW_GLYPH_ID = "raw_glyph_id"
 
 FONT_SCHEMA = cv.Schema(
     {
@@ -83,13 +83,14 @@ FONT_SCHEMA = cv.Schema(
         cv.Optional(CONF_GLYPHS, default=DEFAULT_GLYPHS): validate_glyphs,
         cv.Optional(CONF_SIZE, default=20): cv.int_range(min=1),
         cv.GenerateID(CONF_RAW_DATA_ID): cv.declare_id(cg.uint8),
+        cv.GenerateID(CONF_RAW_GLYPH_ID): cv.declare_id(GlyphData),
     }
 )
 
 CONFIG_SCHEMA = cv.All(validate_pillow_installed, FONT_SCHEMA)
 
 
-def to_code(config):
+async def to_code(config):
     from PIL import ImageFont
 
     path = CORE.relative_config_path(config[CONF_FILE])
@@ -107,7 +108,7 @@ def to_code(config):
         _, (offset_x, offset_y) = font.font.getsize(glyph)
         width, height = mask.size
         width8 = ((width + 7) // 8) * 8
-        glyph_data = [0 for _ in range(height * width8 // 8)]  # noqa: F812
+        glyph_data = [0] * (height * width8 // 8)
         for y in range(height):
             for x in range(width):
                 if not mask.getpixel((x, y)):
@@ -120,8 +121,25 @@ def to_code(config):
     rhs = [HexInt(x) for x in data]
     prog_arr = cg.progmem_array(config[CONF_RAW_DATA_ID], rhs)
 
-    glyphs = []
+    glyph_initializer = []
     for glyph in config[CONF_GLYPHS]:
-        glyphs.append(Glyph(glyph, prog_arr, *glyph_args[glyph]))
+        glyph_initializer.append(
+            cg.StructInitializer(
+                GlyphData,
+                ("a_char", glyph),
+                (
+                    "data",
+                    cg.RawExpression(f"{str(prog_arr)} + {str(glyph_args[glyph][0])}"),
+                ),
+                ("offset_x", glyph_args[glyph][1]),
+                ("offset_y", glyph_args[glyph][2]),
+                ("width", glyph_args[glyph][3]),
+                ("height", glyph_args[glyph][4]),
+            )
+        )
 
-    cg.new_Pvariable(config[CONF_ID], glyphs, ascent, ascent + descent)
+    glyphs = cg.static_const_array(config[CONF_RAW_GLYPH_ID], glyph_initializer)
+
+    cg.new_Pvariable(
+        config[CONF_ID], glyphs, len(glyph_initializer), ascent, ascent + descent
+    )
