@@ -1,6 +1,9 @@
 #include "mqtt_cover.h"
 #include "esphome/core/log.h"
 
+#include "mqtt_const.h"
+
+#ifdef USE_MQTT
 #ifdef USE_COVER
 
 namespace esphome {
@@ -21,7 +24,7 @@ void MQTTCoverComponent::setup() {
   });
   if (traits.get_supports_position()) {
     this->subscribe(this->get_position_command_topic(), [this](const std::string &topic, const std::string &payload) {
-      auto value = parse_float(payload);
+      auto value = parse_number<float>(payload);
       if (!value.has_value()) {
         ESP_LOGW(TAG, "Invalid position value: '%s'", payload.c_str());
         return;
@@ -33,7 +36,7 @@ void MQTTCoverComponent::setup() {
   }
   if (traits.get_supports_tilt()) {
     this->subscribe(this->get_tilt_command_topic(), [this](const std::string &topic, const std::string &payload) {
-      auto value = parse_float(payload);
+      auto value = parse_number<float>(payload);
       if (!value.has_value()) {
         ESP_LOGW(TAG, "Invalid tilt value: '%s'", payload.c_str());
         return;
@@ -61,17 +64,21 @@ void MQTTCoverComponent::dump_config() {
   }
 }
 void MQTTCoverComponent::send_discovery(JsonObject &root, mqtt::SendDiscoveryConfig &config) {
+  if (!this->cover_->get_device_class().empty())
+    root[MQTT_DEVICE_CLASS] = this->cover_->get_device_class();
+
   auto traits = this->cover_->get_traits();
   if (traits.get_is_assumed_state()) {
-    root["optimistic"] = true;
+    root[MQTT_OPTIMISTIC] = true;
   }
   if (traits.get_supports_position()) {
-    root["position_topic"] = this->get_position_state_topic();
-    root["set_position_topic"] = this->get_position_command_topic();
+    config.state_topic = false;
+    root[MQTT_POSITION_TOPIC] = this->get_position_state_topic();
+    root[MQTT_SET_POSITION_TOPIC] = this->get_position_command_topic();
   }
   if (traits.get_supports_tilt()) {
-    root["tilt_status_topic"] = this->get_tilt_state_topic();
-    root["tilt_command_topic"] = this->get_tilt_command_topic();
+    root[MQTT_TILT_STATUS_TOPIC] = this->get_tilt_state_topic();
+    root[MQTT_TILT_COMMAND_TOPIC] = this->get_tilt_command_topic();
   }
   if (traits.get_supports_tilt() && !traits.get_supports_position()) {
     config.command_topic = false;
@@ -79,9 +86,9 @@ void MQTTCoverComponent::send_discovery(JsonObject &root, mqtt::SendDiscoveryCon
 }
 
 std::string MQTTCoverComponent::component_type() const { return "cover"; }
-std::string MQTTCoverComponent::friendly_name() const { return this->cover_->get_name(); }
+const EntityBase *MQTTCoverComponent::get_entity() const { return this->cover_; }
+
 bool MQTTCoverComponent::send_initial_state() { return this->publish_state(); }
-bool MQTTCoverComponent::is_internal() { return this->cover_->is_internal(); }
 bool MQTTCoverComponent::publish_state() {
   auto traits = this->cover_->get_traits();
   bool success = true;
@@ -112,3 +119,4 @@ bool MQTTCoverComponent::publish_state() {
 }  // namespace esphome
 
 #endif
+#endif  // USE_MQTT
